#!/bin/bash
#
# Sinopsis del programa: ./mcd.sh <segundos> <directorio>
#
# Este programa esta diseñado para revisar periodicamente el directorio señalado
# al momento de su ejecucion y enviar un correo electronico automatico (al servidor gmail)
# cuando se detecten cambios en el. 

#--------------------------Variables Globales------------------------------#
# Se guarda el argumento segundos en la variable seg
seg=$1
# Se guarda el argumento directorio en la variable dir
dir=$2
# Se guarda el argumento modificador en la variable dir
flag=$3
# Flag que activa la lectura desde archivo
kflag=ff
# Se crea la variable antiguo que es un archivo de texto llamado old.txt
antiguo=old.txt
# Se crea la variable nuevo que es un archivo de texto llamado new.txt
nuevo=new.txt
# Se crea la variable diferencia que es un archivo de texto llamado diff.txt
diferencia=diff.txt
#--------------------------------------------------------------------------#
#
#
#----------------------------Definicion de funciones-----------------------#
# La funcion new tiene por objetivo escanear el directorio señalado en la ejecucion
function new
{
    # la variable aux es igual a los segundos señalado en la ejecucion
    aux=$3
    # la variable var es igual al directorio señalado en la ejecucion
    var=$2
    # comprueba si la cantidad de segundos es menor al máximo entero con signo
    # posible en una máquina de 32 bit (2^16 -1 )
    if [ $aux -lt 65535 ] 2>> /dev/null ; then
	# comprueba que el directorio es válido
	if [ -d $var ]; then
	    # lista los archivos que se encuentran en el directorio señalado
	    # y se guardan en el archivo temporal new.txt
	    ls -l $var > /tmp/$1
	else
	    # si es que el directorio señalado no es válido entrega un mensaje 
	    # por consola indicando que no es válido
	    echo "el directorio $var no es valido"
	    exit
	fi
    else
	# si es los segundos señalados no son válidos entrega un mensaje por 
	# consola indicando que no es entero válido
	echo "el argumento segundos no es valido: debe ser un numero entero"
	exit
    fi
}

# La funcion comparar tiene por objetivo comparar los dos archivos y 
# mostrar sus diferencias
function comparar
{
    # comprueba si es que hay diferencias entre los dos archivos
    # encuestados y el resultado lo guarda en diff.txt
    # Si es que se producen cambios se retorna un 0 sino se 
    # retorna un 1
    if diff /tmp/$1 /tmp/$2 > /tmp/$3 ; then
	return 0
    else 
	return 1
    fi
}

# La funcion mail tiene por objetivo enviar el mail con la diferencia que se 
# encontro en el directorio supervisado. 
function mail
{
    sleep 1
    # se prueba el servidor de mails
    echo "ehlo testing"
    sleep 1
    # se autentifica el usuario
    echo "auth plain $1"
    sleep 1
    # indica el emisor del mail
    echo "mail from:<$2@gmail.com>"
    sleep 1
    # indica el emisor del mail
    echo "rcpt to:<$2@gmail.com>"
    sleep 1
    # desde aquí empieza el mensaje
    echo "data"
    sleep 1
    # alias para el emisor
    echo "From: Mail Automatico <$2@gmail.com>"
    sleep 1
    # alias para el receptor
    echo "To: Usuario <$2@gmail.com>"
    sleep 1
    # asunto del correo
    echo "Subject: Cambios en directorio $4"
    sleep 1
    # salto de línea pertenece al protocolo para enviar el correo
    echo " "
    sleep 1
    echo  Los cambios que se produjeron en el directorio, "$4, son los siguientes"
    sleep 1
    echo " "
    sleep 1
    # aqui se escriben las diferencias del directorio
    echo "$3"
    sleep 1
    # salto de línea perteneciente al protocolo para enviar el correo
    echo " "
    sleep 1
    # fin del mensaje
    echo "."
    sleep 1
    # comando para salir del servidor de correos
    echo "quit"
}
#--------------------------------------------------------------------------#
#
#
#----------------------------Comienzo del Programa-------------------------#
# 
# entrega un mensaje por pantalla si es que no se ha especificado el primer
# argumento segundos
seg=${seg:?'primer argumento "segundos de ejecucion", no suministrado'}
# entrega un mensaje por pantalla si es que no se ha especificado el segundo
# argumento directorio
dir=${dir:?'segundo argumento "directorio de prueba", no suministrado'}
# invoca a la funcion new con tres argumentos y el resultado se guarda en la
# variable old.txt
new $antiguo $dir $seg

# escribe por pantalla el servidor smtp especificado en el archivo parametros.txt
# y lo guarda en la variable server 

if [ -z $flag ]; then flag=NULL
fi

if [ $flag = $kflag ]; then
  echo Ingrese el servidor smtp a utilizar
  server=`cat parametros.txt | awk -F " " '{print $1}' `
  echo "$server"
  # escribe por pantalla el puerto por el cual se conecta al servidor smtp especificado
  # en el archivo parametros.txt y luego lo guarda en la variable port
  echo Ingrese puerto donde este escucha
  port=`cat parametros.txt | awk -F " " '{print $2}'`
  echo "$port"
  # escribe por pantalla la password codificada en base64 especificado
  # en el archivo parametros.txt y luego lo guarda en la variable pass
  echo Ingrese la password en formato codificado base64 
  pass=`cat parametros.txt | awk -F " " '{print $4}'`
  echo "$pass"
  # escribe por pantalla la cuenta de usuario del servidor especificado
  # en el archivo parametros.txt y luego lo guarda en la variable user
  echo Ingrese la cuenta gmail donde enviará la alerta
  user=`cat parametros.txt | awk -F " " '{print $3}'`
  echo "$user"
else 
  read -p "Ingrese el servidor smtp a utilizar: " server
  # pregunta por consola el puerto por el cual se conecta al servidor smtp especificado
  read -p "Ingrese puerto donde este escucha: " port
  # pregunta por consola la password codificada en base64 especificado
  read -p "Ingrese la password en formato codificado base64: " pass 
  # pregunta por consola la cuenta de usuario del servidor especificado
  read -p "Ingrese la cuenta gmail donde enviará la alerta: " user
fi
sleep 1
clear
echo "En ejecucion..."
while true
do
    # invoca a la funcion new y el resultado lo guarda en la variable 
    # new.txt
    new $nuevo $dir $seg
    # invoca a la funcion comparar y el resultado lo guarda en la variable
    # diff.txt
    comparar $antiguo $nuevo $diferencia
    # si el último retorno que hubo en consola es 1 (funcion comparar) quiere 
    # decir que hubo cambios en el directorio, en cambio, si el último valor es 0 
    # se produce un intervalo de tiempo indicado por el usuario (agumento segundos)
    # para realizar la siguiente encuesta
    if [ $? -ne 1 ] ;then
	sleep $seg
    else
	# se decodifica la password encriptada  en base64 en el archivo parametros.txt
	passdeco=`perl -MMIME::Base64 -e 'print decode_base64("'$pass'")'`
	# se codifica el usuario y la password 
	passdecov2=`perl -MMIME::Base64 -e 'print encode_base64("\000'$user'\@gmail.com\000'$passdeco'")'`
	# se crea la variable mje con la diferencia que encontró en el directorio
	mje=`more /tmp/$diferencia`
	# se invoca a la funcion mail y se conecta al servidor de gmail para luego enviar el 
	# correo electrónico indicando las diferencias que se produjeron en el directorio
	mail $passdecov2 $user "$mje" $dir | openssl s_client -starttls smtp -crlf -connect $server:$port 1> /dev/null
	# se borran los archivos temporales
	rm -f /tmp/$antiguo /tmp/$nuevo /tmp/$diferencia
	exit
    fi
done
#----------------------------------Fin Programa--------------------------------#